import requests

from .context import Context
from .env import CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS


def gql(context: Context, query: str, variables: dict = None) -> dict:
    """Send a GraphQL query to CluedIn.

    Args:
        context (Context): Context object.
        query (str): GraphQL query.
        variables (dict, optional): A dicrionary of variables to be used in the query.
            Defaults to {}.

    Returns:
        dict: JSON response.
    """
    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    if variables is None:
        variables = {}

    json = {
        'query': query,
        'variables': variables
    }

    response = requests.post(
        url=context.gql_api_url,
        json=json,
        headers=headers,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls
    )

    if not response.ok:
        response.raise_for_status()

    return response.json()


def org_gql(context: Context, query: str, variables: dict = None) -> dict:
    """Send a GraphQL query to CluedIn Organization endpoint.

    Args:
        context (Context): Context object.
        query (str): GraphQL query.
        variables (dict, optional): A dicrionary of variables to be used in the query.
            Defaults to {}.

    Returns:
        dict: JSON response.
    """
    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    if variables is None:
        variables = {}

    json = {
        'query': query,
        'variables': variables
    }

    response = requests.post(
        url=context.gql_org_url,
        json=json,
        headers=headers,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls
    )

    if not response.ok:
        response.raise_for_status()

    return response.json()


def entries(context: Context, query: str, variables: dict = None, flat=False) -> list:
    """Get entries from a GraphQL query. This function is a generator.
        It uses the cursor to get the next page of entries.

    Args:
        context (Context): Context object.
        query (str): GraphQL query.
        variables (dict, optional): A dicrionary of variables to be used in the query.

    Returns:
        list: List of entries.

    Yields:
        Iterator[list]: Iterator of entries.
    """

    def flatten_properties(d):
        if 'properties' not in d:
            return d

        for k, v in d['properties'].items():
            if k == 'attribute-type':
                continue

            if k.startswith('property-'):
                k = k[9:]  # len('property-') == 9

            d[k] = v

        del d['properties']

        return d

    if variables is None:
        variables = {}

    response = gql(context, query, variables)

    while ('data' in response and
           'search' in response['data'] and
           'cursor' in response['data']['search'] and
            'entries' in response['data']['search']):

        cursor = response['data']['search']['cursor']

        page_entries = response['data']['search']['entries']

        if len(page_entries) == 0:
            return

        for entry in page_entries:
            if flat:
                entry = flatten_properties(entry)
            yield entry

        variables['cursor'] = cursor

        response = gql(context, query, variables)



import requests

from .context import Context
from .env import CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS

# Account


def get_users(context: Context, org_id: str = None) -> list:
    """Get users for an organization.

    Args:
        context (Context): Context object.
        org_id (str, optional): organization ID. Defaults to None.
            If None, the current user's organization ID is used.

    Returns:
        list: JSON response.
    """
    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    params = {}

    if org_id is not None:
        params['organizationId'] = org_id

    response = requests.get(
        url=f'{context.auth_url}/api/account/accounts',
        headers=headers,
        params=params,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response.json()


# Availability

def is_organization_available_response(context: Context, org_name: str) -> dict:
    """Check if an organization name is available.

    Args:
        context (Context): Context object.
        org_name (str): Organization name.

    Returns:
        dict: JSON response.
    """
    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }
    response = requests.get(
        url=f'{context.auth_url}/api/account/available?clientId={org_name}',
        headers=headers,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response.json()


def is_organization_available(context: Context, org_name: str) -> bool:
    """Check if an organization name is available.

    Args:
        context (Context): Context object.
        org_name (str): Organization name.

    Returns:
        bool: True if available, False otherwise.
    """
    return bool(is_organization_available_response(context, org_name)['isAvailable'])


def is_user_available_response(context: Context, user_email: str, org_name: str) -> dict:
    """Check if a user email is available.

    Args:
        context (Context): Context object.
        user_email (str): User email.
        org_name (str): Organization name.

    Returns:
        dict: JSON response.
    """
    response = requests.get(
        url=f'{context.auth_url}/api/account/username?username={user_email}&clientId={org_name}',
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response.json()


def is_user_available(context: Context, user_email: str, org_name: str) -> bool:
    """Check if a user email is available.

    Args:
        context (Context): Context object.
        user_email (str): User email.
        org_name (str): Organization name.

    Returns:
        bool: True if available, False otherwise.
    """
    return bool(is_user_available_response(context, user_email, org_name)['isAvailable'])


# Registration

def get_invitation_code(context: Context, email: str) -> str:
    """Get an invitation code for a user email.

    Args:
        context (Context): (Context): Context object.
        email (str): User email.

    Returns:
        str: Invitation code.
    """
    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    response = requests.get(
        url=f'{context.auth_url}/api/account/invitationcode?email={email}',
        headers=headers,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    return response.text


def create_organization(
        context: Context,
        user_email: str,
        password: str,
        org_name: str,
        org_sub_domain: str = None,
        email_domain: str = None,
        allow_email_domain_signup: bool = True,
        new_account_access_key: str = None) -> dict:
    """Create an organization with an admin user.

    Args:
        context (Context): Context object.
        user_email (str): Admin user email.
        password (str): Admin user password.
        org_name (str): Organization name.
        org_sub_domain (str, optional): Organization subdomain. Defaults to None. If None, org_name is used.
        email_domain (str, optional): Email domain. Defaults to None. If None, context.domain is used.
        allow_email_domain_signup (bool, optional): Allow email domain sign up. Defaults to True. If True, users can sign up with an email from the email domain only.
        new_account_access_key (str, optional): Used in Kubernetes to protect the endpoint. See `cluedin-new-organization-access-key` secret. Defaults to None.

    Returns:
        dict: JSON response.
    """

    headers = {}

    if new_account_access_key is not None:
        headers['x-cluedin-newaccountaccesskey'] = new_account_access_key

    if org_sub_domain is None:
        org_sub_domain = org_name

    if email_domain is None:
        email_domain = context.domain

    data = {
        'username': user_email,
        'email': user_email,
        'password': password,
        'confirmPassword': password,
        'organizationName': org_name,
        'applicationSubDomain': org_sub_domain,
        'emailDomain': email_domain,
        'grant_type': 'password',
        'allowEmailDomainSignup': allow_email_domain_signup
    }

    response = requests.post(
        url=f'{context.auth_url}/api/account/new',
        headers=headers,
        data=data,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response.json()


def create_user(
        context: Context,
        user_email: str,
        user_password: str,
        application_sub_domain: str = None,
        client_id: str = None) -> requests.models.Response:
    """Create a user.

    Args:
        context (Context): Context object.
        user_email (str): User email.
        user_password (str): User password.
        application_sub_domain (str, optional): Application subdomain. Defaults to None. If None, context.org_name is used.
        client_id (str, optional): ClientId (Organization name). Defaults to None. If None, context.org_name is used.

    Returns:
        requests.models.Response: response object.
    """

    if application_sub_domain is None:
        application_sub_domain = context.org_name

    if client_id is None:
        client_id = context.org_name

    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    invitation_code = get_invitation_code(context, user_email)

    params = {
        'code': invitation_code
    }

    data = {
        'username': user_email,
        'email': user_email,
        'password': user_password,
        'confirmPassword': user_password,
        'applicationSubDomain': application_sub_domain,
        'grant_type': 'password',
        'clientid': client_id
    }

    response = requests.post(
        url=f'{context.auth_url}/api/account/register',
        headers=headers,
        params=params,
        data=data,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response


def create_admin_user(
        context: Context,
        user_email: str,
        user_password: str,
        application_sub_domain: str = None,
        client_id: str = None) -> requests.models.Response:
    """Create an admin user.

    Args:
        context (Context): Context object.
        user_email (str): Admin user email.
        user_password (str): Admin user password.
        application_sub_domain (str, optional): Application subdomain. Defaults to None. If None, context.org_name is used.
        client_id (str, optional): ClientId (Organization name). Defaults to None. If None, context.org_name is used.

    Returns:
        requests.models.Response: response object.
    """

    if application_sub_domain is None:
        application_sub_domain = context.org_name

    if client_id is None:
        client_id = context.org_name

    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    invitation_code = get_invitation_code(context, user_email)

    params = {
        'code': invitation_code
    }

    data = {
        'username': user_email,
        'email': user_email,
        'password': user_password,
        'confirmPassword': user_password,
        'applicationSubDomain': application_sub_domain,
        'grant_type': 'password',
        'clientid': client_id
    }

    response = requests.post(
        url=f'{context.auth_url}/api/account/registeradmin',
        headers=headers,
        params=params,
        data=data,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response


def get_user(context: Context, user_id: str = None) -> dict:
    """Get user.

    Args:
        context (Context): Context object.
        user_id (str, optional): User Id. Defaults to None. If None, the current user is returned.

    Returns:
        dict: JSON response.
    """

    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    params = {}

    if id is not None:
        params['id'] = user_id

    response = requests.get(
        url=f'{context.auth_url}/api/account/user',
        headers=headers,
        params=params,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response.json()



import requests

from .context import Context
from .env import CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS


def get_vocab_keys(context: Context) -> list:
    """Get all vocabulary keys.

    Args:
        context (Context): Context object.

    Returns:
        list: JSON response - list of vocabulary keys.
    """

    headers = {
        'Authorization': f'Bearer {context.access_token}'
    }

    response = requests.get(
        url=f'{context.api_url}/entity/schema',
        headers=headers,
        timeout=CLUEDIN_REQUEST_TIMEOUT_IN_SECONDS,
        verify=context.verify_tls)

    if not response.ok:
        response.raise_for_status()

    return response.json()



from os import environ

from .context import Context

from . import account
from . import context
from . import entity
from . import env
from . import gql
from . import json
from . import jwt
from . import public
from . import vocab



import base64
import json


def get_jwt_payload(jwt: str) -> dict:
    """Get the payload from a JWT.

    Args:
        jwt (str): JWT.

    Returns:
        dict: Payload.
    """
    # Split the token into header, payload, and signature
    _, payload, _ = jwt.split('.')

    # Decode the payload from base64
    decoded_payload = base64.b64decode(payload + '=' * (4 - len(payload) % 4))

    # Convert the decoded payload to string
    return json.loads(decoded_payload.decode('utf-8'))
